package com.hikcreate.DataToHbase

import com.hikcreate.Utils.{DataToHbaseUtils, DateFormatUtils}
import org.apache.hadoop.hbase.client.Put
import org.apache.hadoop.hbase.io.ImmutableBytesWritable
import org.apache.hadoop.hbase.util.Bytes
import org.apache.hadoop.mapreduce.Job
import org.apache.spark.sql.SparkSession
import org.apache.spark.{SparkConf, SparkContext}
import org.elasticsearch.spark.sql.EsSparkSQL

object VehicleToHbase extends Serializable {
  def main(args: Array[String]): Unit = {


    /*val spark: SparkSession = SparkSession.builder()
      .appName("HbasePut")
      .config("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .master("local[*]").getOrCreate()*/

    //告知哪些类型需要序列化
    val conf = new SparkConf()
      .setAppName("toHbase")
      .setMaster("local[*]")
      .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .set("es.nodes","10.197.236.194,10.197.236.201,10.197.236.193")
      .set("es.port","9200")
      .set("es.write.operation","index")
      .registerKryoClasses(Array(classOf[org.apache.hadoop.hbase.io.ImmutableBytesWritable],classOf[DateFormatUtils]))

    val sc = new SparkContext(conf)
    val spark = SparkSession.builder().config(conf).getOrCreate()
    if (args.length < 0) {
      println("传递的参数错误")
      System.exit(1)
    }
    //hbase表名
    val tablename=args(0)

    val job: Job = new DataToHbaseUtils().DataToHbaseConf(tablename)

    val dateUtils=new DateFormatUtils

    import spark.implicits._


    val datacsv = spark.read.format("com.databricks.spark.csv")
      .option("header", "true") //在csv第一行有属性"true"，没有就是"false"
      .option("timestampFormat","yyyy-MM-dd HH:mm:ss")
      .load("/root/vehicle.csv")
      //.load("C:\\Users\\Administrator\\Documents\\vehicle.csv")
      //datacsv.createOrReplaceTempView("vehicle")
      /*val datafmt=spark.sql(
        """
          select
           |XH,
           |HPZL,
           |HPHM,
           |CLPP1,
           |CLXH,
           |CLPP2,
           |GCJK,
           |ZZG,
           |ZZCMC,
           |CLSBDH,
           |FDJH,
           |CLLX,
           |CSYS,
           |SYXZ,
           |SFZMHM,
           |SFZMMC,
           |SYR,
           |SYQ,
           |cast(date_format(to_timestamp(CCDJRQ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as CCDJRQ,
           |cast(date_format(to_timestamp(DJRQ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as DJRQ,
           |cast(date_format(to_timestamp(YXQZ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as YXQZ,
           |cast(date_format(to_timestamp(QZBFQZ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as QZBFQZ,
           |FZJG,
           |GLBM,
           |cast(date_format(to_timestamp(FPRQ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as FPRQ,
           |cast(date_format(to_timestamp(FZRQ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as FZRQ,
           |cast(date_format(to_timestamp(FDJRQ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as FDJRQ,
           |cast(date_format(to_timestamp(FHGZRQ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as FHGZRQ,
           |cast(date_format(to_timestamp(FHGZRQ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as BXZZRQ,
           |BPCS,
           |BZCS,
           |BDJCS,
           |DJZSBH,
           |ZDJZSHS,
           |DABH,
           |XZQH,
           |ZT,
           |DYBJ,
           |JBR,
           |CLLY,
           |LSH,
           |FDJXH,
           |RLZL,
           |PL,
           |GL,
           |ZXXS,
           |CWKC,
           |CWKK,
           |CWKG,
           |HXNBCD,
           |HXNBKD,
           |HXNBGD,
           |GBTHPS,
           |ZS,
           |ZJ,
           |QLJ,
           |HLJ,
           |LTGG,
           |LTS,
           |ZZL,
           |ZBZL,
           |HDZZL,
           |HDZK,
           |ZQYZL,
           |QPZK,
           |HPZK,
           |HBDBQK,
           |cast(date_format(to_timestamp(CCRQ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as CCRQ,
           |HDFS,
           |LLPZ1,
           |PZBH1,
           |LLPZ2,
           |PZBH2,
           |XSDW,
           |XSJG,
           |XSRQ,
           |JKPZ,
           |JKPZHM,
           |HGZBH,
           |NSZM,
           |NSZMBH,
           |cast(date_format(to_timestamp(GXRQ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as GXRQ,
           |XGZL,
           |QMBH,
           |HMBH,
           |BZ,
           |JYW,
           |ZSXZQH,
           |ZSXXDZ,
           |YZBM1,
           |LXDH,
           |ZZZ,
           |ZZXZQH,
           |ZZXXDZ,
           |YZBM2,
           |ZDYZT,
           |YXH,
           |CYRY,
           |DPHGZBH,
           |SQDM,
           |CLYT,
           |YTSX,
           |DZYX,
           |XSZBH,
           |SJHM,
           |JYHGBZBH,
           |DWBH,
           |cast(date_format(to_timestamp(SYQSRQ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as SYQSRQ,
           |cast(date_format(to_timestamp(YQJYQZBFQZ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as YQJYQZBFQZ,
           |cast(date_format(to_timestamp(YQJYQZ2, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as YQJYQZ2,
           |FDJGS,
           |SFYZHGN,
           |ZZJGLX,
           |WXMBC,
           |NCDQSY,
           |cast(date_format(to_timestamp(HPQYSJ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as HPQYSJ,
           |DZBSXLH,
           |SFXNY,
           |XNYZL,
           |QDDJXH,
           |QDDJH,
           |QDDJGL,
           |CNZZZL,
           |CNZZXS,
           |CNZZZDL,
           |CNZZZDY,
           |CNZZDTDY,
           |HDWJCD,
           |CDQDXSLCGK,
           |CDQDXSLCDS,
           |YWJYW
          |from vehicle

        """.stripMargin)*/



    val datacsvDF = datacsv.map(r => {
      Vehicle(
        r.getAs[String]("XH"),
        r.getAs[String]("HPZL"),
        r.getAs[String]("HPHM"),
        r.getAs[String]("CLPP1"),
        r.getAs[String]("CLXH"),
        r.getAs[String]("CLPP2"),
        r.getAs[String]("GCJK"),
        r.getAs[String]("ZZG"),
        r.getAs[String]("ZZCMC"),
        r.getAs[String]("CLSBDH"),
        r.getAs[String]("FDJH"),
        r.getAs[String]("CLLX"),
        r.getAs[String]("CSYS"),
        r.getAs[String]("SYXZ"),
        r.getAs[String]("SFZMHM"),
        r.getAs[String]("SFZMMC"),
        r.getAs[String]("SYR"),
        r.getAs[String]("SYQ"),
        dateUtils.dateFormatRes(r.getAs[String]("CCDJRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("DJRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("YXQZ")),
        dateUtils.dateFormatRes(r.getAs[String]("QZBFQZ")),
        r.getAs[String]("FZJG"),
        r.getAs[String]("GLBM"),
        dateUtils.dateFormatRes(r.getAs[String]("FPRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("FZRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("FDJRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("FHGZRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("BXZZRQ")),
        r.getAs[String]("BPCS"),
        r.getAs[String]("BZCS"),
        r.getAs[String]("BDJCS"),
        r.getAs[String]("DJZSBH"),
        r.getAs[String]("ZDJZSHS"),
        r.getAs[String]("DABH"),
        r.getAs[String]("XZQH"),
        r.getAs[String]("ZT"),
        r.getAs[String]("DYBJ"),
        r.getAs[String]("JBR"),
        r.getAs[String]("CLLY"),
        r.getAs[String]("LSH"),
        r.getAs[String]("FDJXH"),
        r.getAs[String]("RLZL"),
        r.getAs[String]("PL"),
        r.getAs[String]("GL"),
        r.getAs[String]("ZXXS"),
        r.getAs[String]("CWKC"),
        r.getAs[String]("CWKK"),
        r.getAs[String]("CWKG"),
        r.getAs[String]("HXNBCD"),
        r.getAs[String]("HXNBKD"),
        r.getAs[String]("HXNBGD"),
        r.getAs[String]("GBTHPS"),
        r.getAs[String]("ZS"),
        r.getAs[String]("ZJ"),
        r.getAs[String]("QLJ"),
        r.getAs[String]("HLJ"),
        r.getAs[String]("LTGG"),
        r.getAs[String]("LTS"),
        r.getAs[String]("ZZL"),
        r.getAs[String]("ZBZL"),
        r.getAs[String]("HDZZL"),
        r.getAs[String]("HDZK"),
        r.getAs[String]("ZQYZL"),
        r.getAs[String]("QPZK"),
        r.getAs[String]("HPZK"),
        r.getAs[String]("HBDBQK"),
        dateUtils.dateFormatRes(r.getAs[String]("CCRQ")),
        r.getAs[String]("HDFS"),
        r.getAs[String]("LLPZ1"),
        r.getAs[String]("PZBH1"),
        r.getAs[String]("LLPZ2"),
        r.getAs[String]("PZBH2"),
        r.getAs[String]("XSDW"),
        r.getAs[String]("XSJG"),
        r.getAs[String]("XSRQ"),
        r.getAs[String]("JKPZ"),
        r.getAs[String]("JKPZHM"),
        r.getAs[String]("HGZBH"),
        r.getAs[String]("NSZM"),
        r.getAs[String]("NSZMBH"),
        dateUtils.dateFormatRes(r.getAs[String]("GXRQ")),
        r.getAs[String]("XGZL"),
        r.getAs[String]("QMBH"),
        r.getAs[String]("HMBH"),
        r.getAs[String]("BZ"),
        r.getAs[String]("JYW"),
        r.getAs[String]("ZSXZQH"),
        r.getAs[String]("ZSXXDZ"),
        r.getAs[String]("YZBM1"),
        r.getAs[String]("LXDH"),
        r.getAs[String]("ZZZ"),
        r.getAs[String]("ZZXZQH"),
        r.getAs[String]("ZZXXDZ"),
        r.getAs[String]("YZBM2"),
        r.getAs[String]("ZDYZT"),
        r.getAs[String]("YXH"),
        r.getAs[String]("CYRY"),
        r.getAs[String]("DPHGZBH"),
        r.getAs[String]("SQDM"),
        r.getAs[String]("CLYT"),
        r.getAs[String]("YTSX"),
        r.getAs[String]("DZYX"),
        r.getAs[String]("XSZBH"),
        r.getAs[String]("SJHM"),
        r.getAs[String]("JYHGBZBH"),
        r.getAs[String]("DWBH"),
        dateUtils.dateFormatRes(r.getAs[String]("SYQSRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("YQJYQZBFQZ")),
        dateUtils.dateFormatRes(r.getAs[String]("YQJYQZ2")),
        r.getAs[String]("FDJGS"),
        r.getAs[String]("SFYZHGN"),
        r.getAs[String]("ZZJGLX"),
        r.getAs[String]("WXMBC"),
        r.getAs[String]("NCDQSY"),
        dateUtils.dateFormatRes(r.getAs[String]("HPQYSJ")),
        r.getAs[String]("DZBSXLH"),
        r.getAs[String]("SFXNY"),
        r.getAs[String]("XNYZL"),
        r.getAs[String]("QDDJXH"),
        r.getAs[String]("QDDJH"),
        r.getAs[String]("QDDJGL"),
        r.getAs[String]("CNZZZL"),
        r.getAs[String]("CNZZXS"),
        r.getAs[String]("CNZZZDL"),
        r.getAs[String]("CNZZZDY"),
        r.getAs[String]("CNZZDTDY"),
        r.getAs[String]("HDWJCD"),
        r.getAs[String]("CDQDXSLCGK"),
        r.getAs[String]("CDQDXSLCDS"),
        r.getAs[String]("YWJYW")

      )
    }).toDF()


    var columns = datacsvDF.columns
    val kvRDD = datacsvDF.flatMap(row => {
      val rowkey = Bytes.toBytes(row.getAs[String]("xh"))
      columns.map(
        column => {
          val colvalue = if (row.getAs[String](column) == null) "null" else row.getAs[String](column)
          (rowkey, column, colvalue)
        }).toList
    })

    val rerdd = kvRDD.rdd.map(value => {
      val family = Bytes.toBytes("info")
      val key = value._1
      val col = value._2
      val put = new Put(value._1)
      if (!col.isEmpty) {
        put.addColumn(family, Bytes.toBytes(col), Bytes.toBytes(value._3))
      }
      (new ImmutableBytesWritable, put)
    })
    rerdd.take(5).foreach(x => {
      println(x)
    })

    rerdd.saveAsNewAPIHadoopDataset(job.getConfiguration)
    EsSparkSQL.saveToEs(datacsvDF,"vehicle_gy_ys/doc",Map("es.mapping.id"->"xh"))

    sc.stop()

  }

  case class Vehicle(
                      xh: String,
                      hpzl: String,
                      hphm: String,
                      clpp1: String,
                      clxh: String,
                      clpp2: String,
                      gcjk: String,
                      zzg: String,
                      zzcmc: String,
                      clsbdh: String,
                      fdjh: String,
                      cllx: String,
                      csys: String,
                      syxz: String,
                      sfzmhm: String,
                      sfzmmc: String,
                      syr: String,
                      syq: String,
                      ccdjrq: String,
                      djrq: String,
                      yxqz: String,
                      qzbfqz: String,
                      fzjg: String,
                      glbm: String,
                      fprq: String,
                      fzrq: String,
                      fdjrq: String,
                      fhgzrq: String,
                      bxzzrq: String,
                      bpcs: String,
                      bzcs: String,
                      bdjcs: String,
                      djzsbh: String,
                      zdjzshs: String,
                      dabh: String,
                      xzqh: String,
                      zt: String,
                      dybj: String,
                      jbr: String,
                      clly: String,
                      lsh: String,
                      fdjxh: String,
                      rlzl: String,
                      pl: String,
                      gl: String,
                      zxxs: String,
                      cwkc: String,
                      cwkk: String,
                      cwkg: String,
                      hxnbcd: String,
                      hxnbkd: String,
                      hxnbgd: String,
                      gbthps: String,
                      zs: String,
                      zj: String,
                      qlj: String,
                      hlj: String,
                      ltgg: String,
                      lts: String,
                      zzl: String,
                      zbzl: String,
                      hdzzl: String,
                      hdzk: String,
                      zqyzl: String,
                      qpzk: String,
                      hpzk: String,
                      hbdbqk: String,
                      ccrq: String,
                      hdfs: String,
                      llpz1: String,
                      pzbh1: String,
                      llpz2: String,
                      pzbh2: String,
                      xsdw: String,
                      xsjg: String,
                      xsrq: String,
                      jkpz: String,
                      jkpzhm: String,
                      hgzbh: String,
                      nszm: String,
                      nszmbh: String,
                      gxrq: String,
                      xgzl: String,
                      qmbh: String,
                      hmbh: String,
                      bz: String,
                      jyw: String,
                      zsxzqh: String,
                      zsxxdz: String,
                      yzbm1: String,
                      lxdh: String,
                      zzz: String,
                      zzxzqh: String,
                      zzxxdz: String,
                      yzbm2: String,
                      zdyzt: String,
                      yxh: String,
                      cyry: String,
                      dphgzbh: String,
                      sqdm: String,
                      clyt: String,
                      ytsx: String,
                      dzyx: String,
                      xszbh: String,
                      sjhm: String,
                      jyhgbzbh: String,
                      dwbh: String,
                      syqsrq: String,
                      yqjyqzbfqz: String,
                      yqjyqz2: String,
                      fdjgs: String,
                      sfyzhgn: String,
                      zzjglx: String,
                      wxmbc: String,
                      ncdqsy: String,
                      hpqysj: String,
                      dzbsxlh: String,
                      sfxny: String,
                      xnyzl: String,
                      qddjxh: String,
                      qddjh: String,
                      qddjgl: String,
                      cnzzzl: String,
                      cnzzxs: String,
                      cnzzzdl: String,
                      cnzzzdy: String,
                      cnzzdtdy: String,
                      hdwjcd: String,
                      cdqdxslcgk: String,
                      cdqdxslcds: String,
                      ywjyw: String
                    )

}