package com.hikcreate.DataToHbase

/**
  * @Author kaiwang
  * @Date 2021/9/25 15:45
  * @Version 1.0
  *         数据入库到hbase
  */

import java.text.DecimalFormat

import com.hikcreate.Utils.{DataToHbaseUtils, DateFormatUtils}
import org.apache.hadoop.hbase.client.Put
import org.apache.hadoop.hbase.io.ImmutableBytesWritable
import org.apache.hadoop.hbase.util.Bytes
import org.apache.hadoop.mapreduce.Job
import org.apache.spark.sql.SparkSession
import org.apache.spark.{SparkConf, SparkContext}
import org.elasticsearch.spark.sql.EsSparkSQL

object VioViolationToHbase extends Serializable {
  def main(args: Array[String]): Unit = {


    /*val spark: SparkSession = SparkSession.builder()
      .appName("HbasePut")
      .config("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .master("local[*]").getOrCreate()*/

    //告知哪些类型需要序列化
    val conf = new SparkConf()
      .setAppName("toHbase")
      .setMaster("local[*]")
      .set("es.nodes", "10.197.236.194,10.197.236.201,10.197.236.193")
      .set("es.port", "9200")
      .set("es.write.operation", "index")
      .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .registerKryoClasses(Array(classOf[ImmutableBytesWritable], classOf[DateFormatUtils]))

    val sc = new SparkContext(conf)
    val spark = SparkSession.builder().config(conf).getOrCreate()

    if (args.length < 0) {
      println("传递的参数错误")
      System.exit(1)
    }
    //hbase表名
    val tablename=args(0)

    val job: Job = new DataToHbaseUtils().DataToHbaseConf(tablename)

    val dateUtils = new DateFormatUtils

    import spark.implicits._


    val datacsv = spark.read.format("com.databricks.spark.csv")
      .option("header", "true") //在csv第一行有属性"true"，没有就是"false"
      .option("timestampFormat", "yyyy-MM-dd HH:mm:ss")
      .load("/root/VioViolation.csv")
      //.load("C:\\Users\\Administrator\\Documents\\VioViolation.csv")
    //datacsv.createOrReplaceTempView("VioViolation")

    /*val datafmt=spark.sql(
      """
        |select
        |WFBH,
        |JDSLB,
        |JDSBH,
        |WSJYW,
        |RYFL,
        |JSZH,
        |DABH,
        |FZJG,
        |ZJCX,
        |DSR,
        |ZSXZQH,
        |ZSXXDZ,
        |DH,
        |LXFS,
        |CLFL,
        |HPZL,
        |HPHM,
        |JDCSYR,
        |SYXZ,
        |JTFS,
        |cast(date_format(to_timestamp(WFSJ,'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as WFSJ,
        |XZQH,
        |DLLX,
        |GLXZDJ,
        |WFDD,
        |LDDM,
        |DDMS,
        |DDJDWZ,
        |WFDZ,
        |WFXW,
        |WFJFS,
        |FKJE,
        |SCZ,
        |BZZ,
        |ZNJ,
        |ZQMJ,
        |JKFS,
        |FXJG,
        |FXJGMC,
        |CLJG,
        |CLJGMC,
        |CFZL,
        |cast(date_format(to_timestamp(CLSJ,'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as CLSJ,
        |JKBJ,
        |cast(date_format(to_timestamp(JKRQ,'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as JKRQ,
        |PZBH,
        |JSJQBJ,
        |JLLX,
        |LRR,
        |cast(date_format(to_timestamp(LRSJ,'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as LRSJ,
        |JBR1,
        |JBR2,
        |SGDJ,
        |CLDXBJ,
        |JDCCLDXBJ,
        |ZDJLBJ,
        |XXLY,
        |XRMS,
        |DKBJ,
        |JMZNJBJ,
        |ZDBJ,
        |JSJG,
        |FSJG,
        |cast(date_format(to_timestamp(LRSJ,'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as GXSJ,
        |BZ,
        |YWJYW,
        |ZJMC,
        |CCLZRQ,
        |NL,
        |XB,
        |HCBJ,
        |JD,
        |WD,
        |YLZZ1,
        |YLZZ2,
        |YLZZ3,
        |YLZZ4,
        |YLZZ5,
        |YLZZ6,
        |YLZZ7,
        |YLZZ8,
        |CJFS,
        |WFSJ1,
        |WFDD1,
        |LDDM1,
        |DDMS1,
        |JSRXZ,
        |CLYT,
        |XCFW,
        |DZZB,
        |SFZDRY,
        |XYSFZMHM,
        |XYXM,
        |YLZZ11,
        |YLZZ12,
        |YLZZ13,
        |YLZZ14,
        |YLZZ15,
        |YLZZ16,
        |YLZZ17,
        |YLZZ18
        |from VioViolation
      """.stripMargin)*/
    //val df=new DecimalFormat("#.00")

    //入库到hbase的数据schema映射
    val datacsvDF = datacsv.map(r => {
      VioViolation(
        r.getAs[String]("WFBH"),
        r.getAs[String]("JDSLB"),
        r.getAs[String]("JDSBH"),
        r.getAs[String]("WSJYW"),
        r.getAs[String]("RYFL"),
        r.getAs[String]("JSZH"),
        r.getAs[String]("DABH"),
        r.getAs[String]("FZJG"),
        r.getAs[String]("ZJCX"),
        r.getAs[String]("DSR"),
        r.getAs[String]("ZSXZQH"),
        r.getAs[String]("ZSXXDZ"),
        r.getAs[String]("DH"),
        r.getAs[String]("LXFS"),
        r.getAs[String]("CLFL"),
        r.getAs[String]("HPZL"),
        r.getAs[String]("HPHM"),
        r.getAs[String]("JDCSYR"),
        r.getAs[String]("SYXZ"),
        r.getAs[String]("JTFS"),
        dateUtils.dateFormatRes(r.getAs[String]("WFSJ")),
        r.getAs[String]("XZQH"),
        r.getAs[String]("DLLX"),
        r.getAs[String]("GLXZDJ"),
        r.getAs[String]("WFDD"),
        r.getAs[String]("LDDM"),
        r.getAs[String]("DDMS"),
        r.getAs[String]("DDJDWZ"),
        r.getAs[String]("WFDZ"),
        r.getAs[String]("WFXW"),
        r.getAs[String]("WFJFS"),
        r.getAs[String]("FKJE"),
        r.getAs[String]("SCZ"),
        r.getAs[String]("BZZ"),
        r.getAs[String]("ZNJ"),
        r.getAs[String]("ZQMJ"),
        r.getAs[String]("JKFS"),
        r.getAs[String]("FXJG"),
        r.getAs[String]("FXJGMC"),
        r.getAs[String]("CLJG"),
        r.getAs[String]("CLJGMC"),
        r.getAs[String]("CFZL"),
        dateUtils.dateFormatRes(r.getAs[String]("CLSJ")),
        r.getAs[String]("JKBJ"),
        dateUtils.dateFormatRes(r.getAs[String]("JKRQ")),
        r.getAs[String]("PZBH"),
        r.getAs[String]("JSJQBJ"),
        r.getAs[String]("JLLX"),
        r.getAs[String]("LRR"),
        dateUtils.dateFormatRes(r.getAs[String]("LRSJ")),
        r.getAs[String]("JBR1"),
        r.getAs[String]("JBR2"),
        r.getAs[String]("SGDJ"),
        r.getAs[String]("CLDXBJ"),
        r.getAs[String]("JDCCLDXBJ"),
        r.getAs[String]("ZDJLBJ"),
        r.getAs[String]("XXLY"),
        r.getAs[String]("XRMS"),
        r.getAs[String]("DKBJ"),
        r.getAs[String]("JMZNJBJ"),
        r.getAs[String]("ZDBJ"),
        r.getAs[String]("JSJG"),
        r.getAs[String]("FSJG"),
        dateUtils.dateFormatRes(r.getAs[String]("GXSJ")),
        r.getAs[String]("BZ"),
        r.getAs[String]("YWJYW"),
        r.getAs[String]("ZJMC"),
        r.getAs[String]("CCLZRQ"),
        r.getAs[String]("NL"),
        r.getAs[String]("XB"),
        r.getAs[String]("HCBJ"),
        r.getAs[String]("JD"),
        r.getAs[String]("WD"),
        r.getAs[String]("YLZZ1"),
        r.getAs[String]("YLZZ2"),
        r.getAs[String]("YLZZ3"),
        r.getAs[String]("YLZZ4"),
        r.getAs[String]("YLZZ5"),
        r.getAs[String]("YLZZ6"),
        r.getAs[String]("YLZZ7"),
        r.getAs[String]("YLZZ8"),
        r.getAs[String]("CJFS"),
        r.getAs[String]("WFSJ1"),
        r.getAs[String]("WFDD1"),
        r.getAs[String]("LDDM1"),
        r.getAs[String]("DDMS1"),
        r.getAs[String]("JSRXZ"),
        r.getAs[String]("CLYT"),
        r.getAs[String]("XCFW"),
        r.getAs[String]("DZZB"),
        r.getAs[String]("SFZDRY"),
        r.getAs[String]("XYSFZMHM"),
        r.getAs[String]("XYXM"),
        r.getAs[String]("YLZZ11"),
        r.getAs[String]("YLZZ12"),
        r.getAs[String]("YLZZ13"),
        r.getAs[String]("YLZZ14"),
        r.getAs[String]("YLZZ15"),
        r.getAs[String]("YLZZ16"),
        r.getAs[String]("YLZZ17"),
        r.getAs[String]("YLZZ18")
      )
    }).toDF()


    //入库到es数据schema映射
    val datacsvDFLow = datacsv.map(r => {
      VioViolationLow(
        r.getAs[String]("WFBH"),
        r.getAs[String]("JDSLB"),
        r.getAs[String]("JDSBH"),
        r.getAs[String]("WSJYW"),
        r.getAs[String]("RYFL"),
        r.getAs[String]("JSZH"),
        r.getAs[String]("DABH"),
        r.getAs[String]("FZJG"),
        r.getAs[String]("ZJCX"),
        r.getAs[String]("DSR"),
        r.getAs[String]("ZSXZQH"),
        r.getAs[String]("ZSXXDZ"),
        r.getAs[String]("DH"),
        r.getAs[String]("LXFS"),
        r.getAs[String]("CLFL"),
        r.getAs[String]("HPZL"),
        r.getAs[String]("HPHM"),
        r.getAs[String]("JDCSYR"),
        r.getAs[String]("SYXZ"),
        r.getAs[String]("JTFS"),
        dateUtils.dateFormatRes(r.getAs[String]("WFSJ")),
        r.getAs[String]("XZQH"),
        r.getAs[String]("DLLX"),
        r.getAs[String]("GLXZDJ"),
        r.getAs[String]("WFDD"),
        r.getAs[String]("LDDM"),
        r.getAs[String]("DDMS"),
        r.getAs[String]("DDJDWZ"),
        r.getAs[String]("WFDZ"),
        r.getAs[String]("WFXW"),
        r.getAs[String]("WFJFS"),
        r.getAs[String]("FKJE"),
        r.getAs[String]("SCZ"),
        r.getAs[String]("BZZ"),
        r.getAs[String]("ZNJ"),
        r.getAs[String]("ZQMJ"),
        r.getAs[String]("JKFS"),
        r.getAs[String]("FXJG"),
        r.getAs[String]("FXJGMC"),
        r.getAs[String]("CLJG"),
        r.getAs[String]("CLJGMC"),
        r.getAs[String]("CFZL"),
        dateUtils.dateFormatRes(r.getAs[String]("CLSJ")),
        r.getAs[String]("JKBJ"),
        dateUtils.dateFormatRes(r.getAs[String]("JKRQ")),
        r.getAs[String]("PZBH"),
        r.getAs[String]("JSJQBJ"),
        r.getAs[String]("JLLX"),
        r.getAs[String]("LRR"),
        dateUtils.dateFormatRes(r.getAs[String]("LRSJ")),
        r.getAs[String]("JBR1"),
        r.getAs[String]("JBR2"),
        r.getAs[String]("SGDJ"),
        r.getAs[String]("CLDXBJ"),
        r.getAs[String]("JDCCLDXBJ"),
        r.getAs[String]("ZDJLBJ"),
        r.getAs[String]("XXLY"),
        r.getAs[String]("XRMS"),
        r.getAs[String]("DKBJ"),
        r.getAs[String]("JMZNJBJ"),
        r.getAs[String]("ZDBJ"),
        r.getAs[String]("JSJG"),
        r.getAs[String]("FSJG"),
        dateUtils.dateFormatRes(r.getAs[String]("GXSJ")),
        r.getAs[String]("BZ"),
        r.getAs[String]("YWJYW"),
        r.getAs[String]("ZJMC"),
        r.getAs[String]("CCLZRQ"),
        r.getAs[String]("NL"),
        r.getAs[String]("XB"),
        r.getAs[String]("HCBJ"),
        r.getAs[String]("JD"),
        r.getAs[String]("WD"),
        r.getAs[String]("YLZZ1"),
        r.getAs[String]("YLZZ2"),
        r.getAs[String]("YLZZ3"),
        r.getAs[String]("YLZZ4"),
        r.getAs[String]("YLZZ5"),
        r.getAs[String]("YLZZ6"),
        r.getAs[String]("YLZZ7"),
        r.getAs[String]("YLZZ8"),
        r.getAs[String]("CJFS"),
        r.getAs[String]("WFSJ1"),
        r.getAs[String]("WFDD1"),
        r.getAs[String]("LDDM1"),
        r.getAs[String]("DDMS1"),
        r.getAs[String]("JSRXZ"),
        r.getAs[String]("CLYT"),
        r.getAs[String]("XCFW"),
        r.getAs[String]("DZZB"),
        r.getAs[String]("SFZDRY"),
        r.getAs[String]("XYSFZMHM"),
        r.getAs[String]("XYXM"),
        r.getAs[String]("YLZZ11"),
        r.getAs[String]("YLZZ12"),
        r.getAs[String]("YLZZ13"),
        r.getAs[String]("YLZZ14"),
        r.getAs[String]("YLZZ15"),
        r.getAs[String]("YLZZ16"),
        r.getAs[String]("YLZZ17"),
        r.getAs[String]("YLZZ18")
      )
    }).toDF()

    //string字段类型转成long类型
    datacsvDFLow.createOrReplaceTempView("violation")
    val dataFmt = spark.sql(
      """
        |select wfbh,
        |jdslb,
        |jdsbh,
        |wsjyw,
        |ryfl,
        |jszh,
        |dabh,
        |fzjg,
        |zjcx,
        |dsr,
        |zsxzqh,
        |zsxxdz,
        |dh,
        |lxfs,
        |clfl,
        |hpzl,
        |hphm,
        |jdcsyr,
        |syxz,
        |jtfs,
        |wfsj,
        |xzqh,
        |dllx,
        |glxzdj,
        |wfdd,
        |lddm,
        |ddms,
        |ddjdwz,
        |wfdz,
        |wfxw,
        |wfjfs,
        |fkje,
        |scz,
        |bzz,
        |cast(znj as Long)znj,
        |zqmj,
        |jkfs,
        |fxjg,
        |fxjgmc,
        |cljg,
        |cljgmc,
        |cfzl,
        |clsj,
        |jkbj,
        |jkrq,
        |pzbh,
        |jsjqbj,
        |jllx,
        |lrr,
        |lrsj,
        |jbr1,
        |jbr2,
        |sgdj,
        |cldxbj,
        |jdccldxbj,
        |zdjlbj,
        |xxly,
        |xrms,
        |dkbj,
        |jmznjbj,
        |zdbj,
        |jsjg,
        |fsjg,
        |gxsj,
        |bz,
        |ywjyw,
        |zjmc,
        |cclzrq,
        |nl,
        |xb,
        |hcbj,
        |jd,
        |wd,
        |ylzz1,
        |ylzz2,
        |ylzz3,
        |ylzz4,
        |ylzz5,
        |ylzz6,
        |ylzz7,
        |ylzz8,
        |cjfs,
        |wfsj1,
        |wfdd1,
        |lddm1,
        |ddms1,
        |jsrxz,
        |clyt,
        |xcfw,
        |dzzb,
        |sfzdry,
        |xysfzmhm,
        |xyxm,
        |ylzz11,
        |ylzz12,
        |ylzz13,
        |ylzz14,
        |ylzz15,
        |ylzz16,
        |ylzz17,
        |ylzz18
        |from violation
      """.stripMargin)


    var columns = datacsvDF.columns
    val kvRDD = datacsvDF.flatMap(row => {
      val rowkey = Bytes.toBytes(row.getAs[String]("WFBH"))
      columns.map(
        column => {
          val colvalue = if (row.getAs[String](column) == null) "null" else row.getAs[String](column)
          (rowkey, column, colvalue)
        }).toList
    })

    val rerdd = kvRDD.rdd.map(value => {
      val family = Bytes.toBytes("info")
      val key = value._1
      val col = value._2
      val put = new Put(value._1)
        put.addColumn(family, Bytes.toBytes(col), Bytes.toBytes(value._3))

      (new ImmutableBytesWritable, put)
    })
    rerdd.take(5).foreach(x => {
      println(x)
    })

    rerdd.saveAsNewAPIHadoopDataset(job.getConfiguration)
    EsSparkSQL.saveToEs(dataFmt,"violation_ys/doc",Map("es.mapping.id"->"wfbh"))
    sc.stop()
  }

    case class VioViolation(WFBH: String,
                            JDSLB: String,
                            JDSBH: String,
                            WSJYW: String,
                            RYFL: String,
                            JSZH: String,
                            DABH: String,
                            FZJG: String,
                            ZJCX: String,
                            DSR: String,
                            ZSXZQH: String,
                            ZSXXDZ: String,
                            DH: String,
                            LXFS: String,
                            CLFL: String,
                            HPZL: String,
                            HPHM: String,
                            JDCSYR: String,
                            SYXZ: String,
                            JTFS: String,
                            WFSJ: String,
                            XZQH: String,
                            DLLX: String,
                            GLXZDJ: String,
                            WFDD: String,
                            LDDM: String,
                            DDMS: String,
                            DDJDWZ: String,
                            WFDZ: String,
                            WFXW: String,
                            WFJFS: String,
                            FKJE: String,
                            SCZ: String,
                            BZZ: String,
                            ZNJ: String,
                            ZQMJ: String,
                            JKFS: String,
                            FXJG: String,
                            FXJGMC: String,
                            CLJG: String,
                            CLJGMC: String,
                            CFZL: String,
                            CLSJ: String,
                            JKBJ: String,
                            JKRQ: String,
                            PZBH: String,
                            JSJQBJ: String,
                            JLLX: String,
                            LRR: String,
                            LRSJ: String,
                            JBR1: String,
                            JBR2: String,
                            SGDJ: String,
                            CLDXBJ: String,
                            JDCCLDXBJ: String,
                            ZDJLBJ: String,
                            XXLY: String,
                            XRMS: String,
                            DKBJ: String,
                            JMZNJBJ: String,
                            ZDBJ: String,
                            JSJG: String,
                            FSJG: String,
                            GXSJ: String,
                            BZ: String,
                            YWJYW: String,
                            ZJMC: String,
                            CCLZRQ: String,
                            NL: String,
                            XB: String,
                            HCBJ: String,
                            JD: String,
                            WD: String,
                            YLZZ1: String,
                            YLZZ2: String,
                            YLZZ3: String,
                            YLZZ4: String,
                            YLZZ5: String,
                            YLZZ6: String,
                            YLZZ7: String,
                            YLZZ8: String,
                            CJFS: String,
                            WFSJ1: String,
                            WFDD1: String,
                            LDDM1: String,
                            DDMS1: String,
                            JSRXZ: String,
                            CLYT: String,
                            XCFW: String,
                            DZZB: String,
                            SFZDRY: String,
                            XYSFZMHM: String,
                            XYXM: String,
                            YLZZ11: String,
                            YLZZ12: String,
                            YLZZ13: String,
                            YLZZ14: String,
                            YLZZ15: String,
                            YLZZ16: String,
                            YLZZ17: String,
                            YLZZ18: String)

case class VioViolationLow(wfbh: String,
                             jdslb: String,
                             jdsbh: String,
                             wsjyw: String,
                             ryfl: String,
                             jszh: String,
                             dabh: String,
                             fzjg: String,
                             zjcx: String,
                             dsr: String,
                             zsxzqh: String,
                             zsxxdz: String,
                             dh: String,
                             lxfs: String,
                             clfl: String,
                             hpzl: String,
                             hphm: String,
                             jdcsyr: String,
                             syxz: String,
                             jtfs: String,
                             wfsj: String,
                             xzqh: String,
                             dllx: String,
                             glxzdj: String,
                             wfdd: String,
                             lddm: String,
                             ddms: String,
                             ddjdwz: String,
                             wfdz: String,
                             wfxw: String,
                             wfjfs: String,
                             fkje: String,
                             scz: String,
                             bzz: String,
                             znj: String,
                             zqmj: String,
                             jkfs: String,
                             fxjg: String,
                             fxjgmc: String,
                             cljg: String,
                             cljgmc: String,
                             cfzl: String,
                             clsj: String,
                             jkbj: String,
                             jkrq: String,
                             pzbh: String,
                             jsjqbj: String,
                             jllx: String,
                             lrr: String,
                             lrsj: String,
                             jbr1: String,
                             jbr2: String,
                             sgdj: String,
                             cldxbj: String,
                             jdccldxbj: String,
                             zdjlbj: String,
                             xxly: String,
                             xrms: String,
                             dkbj: String,
                             jmznjbj: String,
                             zdbj: String,
                             jsjg: String,
                             fsjg: String,
                             gxsj: String,
                             bz: String,
                             ywjyw: String,
                             zjmc: String,
                             cclzrq: String,
                             nl: String,
                             xb: String,
                             hcbj: String,
                             jd: String,
                             wd: String,
                             ylzz1: String,
                             ylzz2: String,
                             ylzz3: String,
                             ylzz4: String,
                             ylzz5: String,
                             ylzz6: String,
                             ylzz7: String,
                             ylzz8: String,
                             cjfs: String,
                             wfsj1: String,
                             wfdd1: String,
                             lddm1: String,
                             ddms1: String,
                             jsrxz: String,
                             clyt: String,
                             xcfw: String,
                             dzzb: String,
                             sfzdry: String,
                             xysfzmhm: String,
                             xyxm: String,
                             ylzz11: String,
                             ylzz12: String,
                             ylzz13: String,
                             ylzz14: String,
                             ylzz15: String,
                             ylzz16: String,
                             ylzz17: String,
                             ylzz18: String)
}

