package com.hikcreate.DataToHbase

import com.hikcreate.Utils.{DataToHbaseUtils, DateFormatUtils}
import org.apache.hadoop.hbase.client.Put
import org.apache.hadoop.hbase.io.ImmutableBytesWritable
import org.apache.hadoop.hbase.util.Bytes
import org.apache.hadoop.mapreduce._
import org.apache.spark.sql.SparkSession
import org.apache.spark.{SparkConf, SparkContext}


object WriteToHbase extends Serializable {

  def main(args: Array[String]): Unit = {
    //告知哪些类型需要序列化
    val conf = new SparkConf()
      .setAppName("toHbase")
      .setMaster("local[*]")
      .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .registerKryoClasses(Array(classOf[ImmutableBytesWritable],classOf[DateFormatUtils]))


    val sc = new SparkContext(conf)
    val spark = SparkSession.builder().config(conf).getOrCreate()

    if (args.length < 0) {
      println("传递的参数错误")
      System.exit(1)
    }
    //hbase表名
    val tablename=args(0)

    val job:Job =new DataToHbaseUtils().DataToHbaseConf(tablename)

    val dateUtils=new DateFormatUtils

    import spark.implicits._


    val datacsv = spark.read.format("com.databricks.spark.csv")
      .option("header", "true") //在csv第一行有属性"true"，没有就是"false"
      //.load("C:\\Users\\Administrator\\Documents\\drivinglicense.csv")
    .load("/root/drivinglicense.csv")

    //datacsv.createOrReplaceTempView("drivinglicense")

    /*val datafmt=spark.sql(
      """
           |select
           |DABH,
           |SFZMHM,
           |ZJCX,
           |YZJCX,
           |cast(date_format(to_timestamp(QFRQ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as QFRQ,
           |cast(date_format(to_timestamp(SYRQ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as SYRQ,
           |cast(date_format(to_timestamp(CCLZRQ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as CCLZRQ,
           |CCFZJG,
           |JZQX,
           |cast(date_format(to_timestamp(YXQS, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as YXQS,
           |cast(date_format(to_timestamp(YXQZ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as YXQZ,
           |LJJF,
           |CFRQ,
           |XXTZRQ,
           |BZCS,
           |ZT,
           |LY,
           |JXMC,
           |JLY,
           |XZQH,
           |XZQJ,
           |cast(date_format(to_timestamp(FZRQ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as FZRQ,
           |JBR,
           |GLBM,
           |FZJG,
           |cast(date_format(to_timestamp(GXSJ, 'yyyy/MM/dd HH:mm:ss'), 'yyyy-MM-dd HH:mm:ss') as string) as GXSJ,
           |LSH,
           |XGZL,
           |BZ,
           |JYW,
           |YDABH,
           |SQDM,
           |ZXBH,
           |XH,
           |SFZMMC,
           |HMCD,
           |XM,
           |XB,
           |cast(date_format(to_timestamp(CSRQ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as CSRQ,
           |GJ,
           |DJZSXZQH,
           |DJZSXXDZ,
           |LXZSXZQH,
           |LXZSXXDZ,
           |LXZSYZBM,
           |LXDH,
           |SJHM,
           |DZYX,
           |ZZZM,
           |ZZFZJG,
           |ZZFZRQ,
           |SFBD,
           |DWBH,
           |cast(date_format(to_timestamp(SYYXQZ, 'yyyy/MM/dd'), 'yyyy-MM-dd') as string) as SYYXQZ,
           |XCZG,
           |XCZJCX,
           |RYZT
           |from drivinglicense

        """.stripMargin)*/
    //datafmt.show()

    val datacsvDF = datacsv.map(r => {
      Drivinglicense(
        r.getAs[String]("DABH"),
        r.getAs[String]("SFZMHM"),
        r.getAs[String]("ZJCX"),
        r.getAs[String]("YZJCX"),
        dateUtils.dateFormatRes(r.getAs[String]("QFRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("SYRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("CCLZRQ")),
        r.getAs[String]("CCFZJG"),
        r.getAs[String]("JZQX"),
        dateUtils.dateFormatRes(r.getAs[String]("YXQS")),
        dateUtils.dateFormatRes(r.getAs[String]("YXQZ")),
        r.getAs[String]("LJJF"),
        dateUtils.dateFormatRes(r.getAs[String]("CFRQ")),
        dateUtils.dateFormatRes(r.getAs[String]("XXTZRQ")),
        r.getAs[String]("BZCS"),
        r.getAs[String]("ZT"),
        r.getAs[String]("LY"),
        r.getAs[String]("JXMC"),
        r.getAs[String]("JLY"),
        r.getAs[String]("XZQH"),
        r.getAs[String]("XZQJ"),
        dateUtils.dateFormatRes(r.getAs[String]("FZRQ")),
        r.getAs[String]("JBR"),
        r.getAs[String]("GLBM"),
        r.getAs[String]("FZJG"),
        dateUtils.dateFormatRes(r.getAs[String]("GXSJ")),
        r.getAs[String]("LSH"),
        r.getAs[String]("XGZL"),
        r.getAs[String]("BZ"),
        r.getAs[String]("JYW"),
        r.getAs[String]("YDABH"),
        r.getAs[String]("SQDM"),
        r.getAs[String]("ZXBH"),
        r.getAs[String]("XH"),
        r.getAs[String]("SFZMMC"),
        r.getAs[String]("HMCD"),
        r.getAs[String]("XM"),
        r.getAs[String]("XB"),
        dateUtils.dateFormatRes(r.getAs[String]("CSRQ")),
        r.getAs[String]("GJ"),
        r.getAs[String]("DJZSXZQH"),
        r.getAs[String]("DJZSXXDZ"),
        r.getAs[String]("LXZSXZQH"),
        r.getAs[String]("LXZSXXDZ"),
        r.getAs[String]("LXZSYZBM"),
        r.getAs[String]("LXDH"),
        r.getAs[String]("SJHM"),
        r.getAs[String]("DZYX"),
        r.getAs[String]("ZZZM"),
        r.getAs[String]("ZZFZJG"),
        r.getAs[String]("ZZFZRQ"),
        r.getAs[String]("SFBD"),
        r.getAs[String]("DWBH"),
        dateUtils.dateFormatRes(r.getAs[String]("SYYXQZ")),
        r.getAs[String]("XCZG"),
        r.getAs[String]("XCZJCX"),
        r.getAs[String]("RYZT")
      )
    }).toDF()


    var columns = datacsvDF.columns
    val kvRDD = datacsvDF.flatMap(row => {
      val rowkey = Bytes.toBytes(row.getAs[String]("SFZMHM"))
      columns.map(
        column => {
          val colvalue = if(row.getAs[String](column) ==null) "null" else row.getAs[String](column)
          (rowkey, column, colvalue)
        }).toList
    })

      val rerdd=kvRDD.rdd.map(value => {
        val family=Bytes.toBytes("info")
        val key=value._1
        val col=value._2
        val put = new Put(value._1)
        if(!col.isEmpty) {
          put.addColumn(family, Bytes.toBytes(col),Bytes.toBytes(value._3))
        }
        (new ImmutableBytesWritable, put)
      })
      rerdd.take(5).foreach(x=>{println(x)})

      rerdd.saveAsNewAPIHadoopDataset(job.getConfiguration)

    sc.stop()

  }



  case class Drivinglicense(DABH: String,
                            SFZMHM: String,
                            ZJCX: String,
                            YZJCX: String,
                            QFRQ: String,
                            SYRQ: String,
                            CCLZRQ: String,
                            CCFZJG: String,
                            JZQX: String,
                            YXQS: String,
                            YXQZ: String,
                            LJJF: String,
                            CFRQ: String,
                            XXTZRQ: String,
                            BZCS: String,
                            ZT: String,
                            LY: String,
                            JXMC: String,
                            JLY: String,
                            XZQH: String,
                            XZQJ: String,
                            FZRQ: String,
                            JBR: String,
                            GLBM: String,
                            FZJG: String,
                            GXSJ: String,
                            LSH: String,
                            XGZL: String,
                            BZ: String,
                            JYW: String,
                            YDABH: String,
                            SQDM: String,
                            ZXBH: String,
                            XH: String,
                            SFZMMC: String,
                            HMCD: String,
                            XM: String,
                            XB: String,
                            CSRQ: String,
                            GJ: String,
                            DJZSXZQH: String,
                            DJZSXXDZ: String,
                            LXZSXZQH: String,
                            LXZSXXDZ: String,
                            LXZSYZBM: String,
                            LXDH: String,
                            SJHM: String,
                            DZYX: String,
                            ZZZM: String,
                            ZZFZJG: String,
                            ZZFZRQ: String,
                            SFBD: String,
                            DWBH: String,
                            SYYXQZ: String,
                            XCZG: String,
                            XCZJCX: String,
                            RYZT: String
                           )


}